package cn.ezhizao.common.utils.collections;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 集合的工具类
 * @author 朱汕
 */
public class CollectionUtils {

    /** 获取集合中的重复元素 */
    public static <E> List<E> getDuplicateElements(List<E> list) {
        // list 对应的 Stream
        return list.stream()
                // 获得元素出现频率的 Map，键为元素，值为元素出现的次数
                .collect(Collectors.toMap(e -> e, e -> 1, Integer::sum))
                // 所有 entry 对应的 Stream
                .entrySet().stream()
                // 过滤出元素出现次数大于 1 的 entry
                .filter(entry -> entry.getValue() > 1)
                // 获得 entry 的键（重复元素）对应的 Stream
                .map(Map.Entry::getKey)
                // 转化为 List
                .collect(Collectors.toList());
    }

}
