package cn.ezhizao.framework.web.controller;

import cn.hutool.core.util.ReflectUtil;
import com.baomidou.mybatisplus.extension.service.IService;
import cn.ezhizao.common.constant.HttpStatus;
import cn.ezhizao.common.utils.DateUtils;
import cn.ezhizao.common.utils.PageUtils;
import cn.ezhizao.common.utils.SecurityUtils;
import cn.ezhizao.common.utils.StringUtils;
import cn.ezhizao.common.utils.sql.SqlUtil;
import cn.ezhizao.framework.security.LoginUser;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.framework.web.page.PageDomain;
import cn.ezhizao.framework.web.page.TableDataInfo;
import cn.ezhizao.framework.web.page.TableSupport;
import cn.ezhizao.project.system.domain.SysUser;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.InitBinder;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.beans.PropertyEditorSupport;
import java.lang.reflect.Field;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * web层通用数据处理
 *
 * @author ruoyi
 */
public class BaseController {
    protected final Logger logger = LoggerFactory.getLogger(this.getClass());

    @Resource
    private HttpServletRequest request;
    /**
     * 将前台传递过来的日期格式的字符串，自动转化为Date类型
     */
    @InitBinder
    public void initBinder(WebDataBinder binder) {
        // Date 类型转换
        binder.registerCustomEditor(Date.class, new PropertyEditorSupport() {
            @Override
            public void setAsText(String text) {
                setValue(DateUtils.parseDate(text));
            }
        });
    }

    /**
     * 设置请求分页数据
     */
    protected void startPage() {
        PageUtils.startPage();
    }

    /**
     * 设置请求排序数据
     */
    protected void startOrderBy() {
        PageDomain pageDomain = TableSupport.buildPageRequest();
        if (StringUtils.isNotEmpty(pageDomain.getOrderBy())) {
            String orderBy = SqlUtil.escapeOrderBySql(pageDomain.getOrderBy());
            PageHelper.orderBy(orderBy);
        }
    }

    /**
     * 清理分页的线程变量
     */
    protected void clearPage() {
        PageUtils.clearPage();
    }

    /**
     * 响应请求分页数据
     */
    @SuppressWarnings({"rawtypes", "unchecked"})
    protected TableDataInfo getDataTable(List<?> list) {
        TableDataInfo rspData = new TableDataInfo();
        rspData.setCode(HttpStatus.SUCCESS);
        rspData.setMsg("查询成功");
        rspData.setRows(list);
        rspData.setTotal(new PageInfo(list).getTotal());
        return rspData;
    }

    /**
     * 生成TableDataInfo对象
     *
     * @param list      查询结果集合
     * @param otherList 其他结果集合
     * @param total     总数
     * @param msg       结果信息
     * @return TableDataInfo对象
     */
    protected TableDataInfo getDataTable(List<?> list, List<?> otherList, Integer total, String msg) {
        TableDataInfo rspData = new TableDataInfo();
        rspData.setCode(HttpStatus.SUCCESS);
        rspData.setMsg("查询成功");
        rspData.setRows(list);
        rspData.setTotal(total);
        rspData.setMsg(msg);
        return rspData;
    }


    /**
     * 生成TableDataInfo对象
     *
     * @param list  查询结果集合
     * @param total 总数
     * @param msg   消息
     * @return TableDataInfo对象
     */
    protected TableDataInfo getDataTable(List<?> list, Integer total, String msg) {
        TableDataInfo rspData = new TableDataInfo();
        rspData.setCode(HttpStatus.SUCCESS);
        rspData.setMsg("查询成功");
        rspData.setRows(list);
        rspData.setTotal(total);
        rspData.setMsg(msg);
        return rspData;
    }


    /**
     * 返回成功
     */
    public AjaxResult success() {
        return AjaxResult.success();
    }

    /**
     * 返回成功消息
     */
    public AjaxResult success(String message) {
        return AjaxResult.success(message);
    }

    /**
     * 返回成功消息
     */
    public AjaxResult success(Object data) {
        return AjaxResult.success(data);
    }

    /**
     * 返回失败消息
     */
    public AjaxResult error() {
        return AjaxResult.error();
    }

    /**
     * 返回失败消息
     */
    public AjaxResult error(String message) {
        return AjaxResult.error(message);
    }

    /**
     * 返回警告消息
     */
    public AjaxResult warn(String message) {
        return AjaxResult.warn(message);
    }

    /**
     * 响应返回结果
     *
     * @param rows 影响行数
     * @return 操作结果
     */
    protected AjaxResult toAjax(int rows) {
        return rows > 0 ? AjaxResult.success() : AjaxResult.error();
    }

    /**
     * 响应返回结果
     *
     * @param result 结果
     * @return 操作结果
     */
    protected AjaxResult toAjax(boolean result) {
        return result ? success() : error();
    }

    /**
     * 获取用户缓存信息
     */
    public LoginUser getLoginUser() {
        return SecurityUtils.getLoginUser();
    }

    /**
     * 获取登录用户id
     */
    public Long getUserId() {
        return getLoginUser().getUserId();
    }

    /**
     * 获取登录部门id
     */
    public Long getDeptId() {
        return getLoginUser().getDeptId();
    }

    /**
     * 获取登录用户名
     */
    public String getUsername() {
        return getLoginUser().getUsername();
    }


    /**
     * 利用反射 统一设置 当前登录用户 的companyId 作为查询条件
     */
    protected <T> void setTenantId(T t) throws NoSuchFieldException, IllegalAccessException {
        SysUser user = SecurityUtils.getLoginUser().getUser();
        Long tenantId = request.getHeader("tenantId") != null && !Objects.equals(request.getHeader("tenantId"), "0") ? Long.parseLong(request.getHeader("tenantId")) : null;
        if (!user.isAdmin()) {
            Field field = t.getClass().getDeclaredField("tenantId");
            field.setAccessible(true);
            field.set(t, tenantId);
        }
    }

    protected Long getTenantId() {
        return request.getHeader("tenantId") != null && !Objects.equals(request.getHeader("tenantId"), "0") ? Long.parseLong(request.getHeader("tenantId")) : null;
    }

    protected <T> boolean saveOrUpdateBatch(IService<T> service, List<T> newList, List<T> oldList) {
        // 批量保存/跟新 新数据 避免使用 saveOrUpdate() 方法 数据量超过50条就很卡了
        boolean bool =
                service.updateBatchById(newList.stream().filter(l -> getId(l) != null).collect(Collectors.toList()))
                        |
                        service.saveBatch(newList.stream().filter(l -> getId(l) == null).collect(Collectors.toList()));
        // 删除差异项目
        if (bool) {
            service.removeBatchByIds(getDifferentList(newList, oldList));
        }
        return bool;
    }

    /**
     * 获取差异数组
     */
    private <T> List<T> getDifferentList(List<T> newList, List<T> oldList) {
        return oldList.stream()
                .filter(t1 -> newList.stream()
                        .noneMatch(t2 -> getId(t2).equals(getId(t1))))
                .collect(Collectors.toList());
    }

    /**
     * 获取 id 属性值
     */
    private <T> Long getId(T t) {
        return (Long) ReflectUtil.getFieldValue(t, "id");
    }


    /**
     * 事务回滚 结合  @Transactional(rollbackFor = RuntimeException.class) 注解使用
     */
    protected void rollBack() {
        TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
    }
}
