package cn.ezhizao.framework.web.controller;


import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.hutool.core.util.ReUtil;
import cn.hutool.http.HttpRequest;
import cn.ezhizao.common.constant.HuaWeiConstant;
import cn.ezhizao.common.constant.Validator;
import cn.ezhizao.framework.redis.RedisCache;
import cn.ezhizao.framework.web.domain.AjaxResult;
import io.swagger.annotations.Api;
import org.apache.commons.codec.binary.Hex;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.http.HttpHeaders;
import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.methods.RequestBuilder;
import org.apache.http.client.utils.URLEncodedUtils;
import org.apache.http.conn.ssl.DefaultHostnameVerifier;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.ssl.SSLContextBuilder;
import org.apache.http.util.EntityUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.nio.charset.Charset;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @author 朱汕
 */
@Api(tags={"短信服务接口"})
@RequestMapping("/sms")
@RestController
public class SendSmsController extends BaseController {
    @Resource
    RedisCache redisCache;
    /** 无需修改,用于格式化鉴权头域,给"X-WSSE"参数赋值 */
    private static final String WSSE_HEADER_FORMAT = "UsernameToken Username=\"%s\",PasswordDigest=\"%s\",Nonce=\"%s\",Created=\"%s\"";
    /** 无需修改,用于格式化鉴权头域,给"Authorization"参数赋值 */
    private static final String AUTH_HEADER_VALUE = "WSSE realm=\"SDP\",profile=\"UsernameToken\",type=\"Appkey\"";

    @PostMapping("/sendSms")
    @Log(title = "发送Sms短信", businessType = BusinessType.OTHER)
    public AjaxResult sendSms(@RequestBody Map<String,Object> map) throws Exception {
        // 必填,请参考"开发准备"获取如下数据,替换为实际值
        // APP接入地址+接口访问URI
        /*必填,全局号码格式(包含国家码),示例:+8615123456789,多个号码之间用英文逗号分隔*/
        String receiver = "+86";
        String phoneNum = map.get("phone_number")==null?null:map.get("phone_number").toString();
        if(phoneNum==null|| ReUtil.isMatch(Validator.REGEX_MOBILE,phoneNum)){
            receiver = receiver+phoneNum;
        }else{
            return error("请输入正确的手机号!");
        }
        if(redisCache.hasKey(phoneNum)){
            return error("请勿频繁操作!");
        }else {
            String url = "https://smsapi.cn-north-4.myhuaweicloud.com:443/sms/batchSendSms/v1";
            // 应用管理中的 APP_Key
            String appKey = HuaWeiConstant.APP_KEY;
            //应用管理中的APP_Secret
            String appSecret = HuaWeiConstant.APP_SECRET;
            //签名管理中的国内短信签名通道号或国际/港澳台短信通道号
            String sender = HuaWeiConstant.APP_SENDER;
            //模板管理中的模板ID
            String templateId = HuaWeiConstant.APP_TEMPLATE_ID;

        /* 条件必填,国内短信关注,当templateId指定的模板类型为通用模板时生效且必填,必须是已审核通过的,与模板类型一致的签名名称
           国际/港澳台短信不用关注该参数
           签名名称  */
            String signature = "华为云短信验证码";

            /*选填,短信状态报告接收地址,推荐使用域名,为空或者不填表示不接收状态报告*/
            String statusCallBack = "";

            /**
             * 选填,使用无变量模板时请赋空值 String templateParas = "";
             * 单变量模板示例:模板内容为"您的验证码是${1}"时,templateParas可填写为"[\"369751\"]"
             * 双变量模板示例:模板内容为"您有${1}件快递请到${2}领取"时,templateParas可填写为"[\"3\",\"人民公园正门\"]"
             * 模板中的每个变量都必须赋值，且取值不能为空
             * 查看更多模板和变量规范:产品介绍>模板和变量规范
             */
            /**
             * 随机生成六位数的验证码代码样例
             */
            String code = "";
            Random random = new Random();
            for (int i = 0; i < 6; i++) {
                int r = random.nextInt(10);
                code = code + r;
            }
            // 模板变量，此处以单变量验证码短信为例，请客户自行生成6位验证码，并定义为字符串类型，以杜绝首位0丢失的问题（例如：002569变成了2569）。
            String templateParas = "[\"" + code + "\"]";
            // 将生成的验证码存到redis中,5分钟有效
            redisCache.setCacheObject(phoneNum, code);
            redisCache.expire(phoneNum,300L);
            //请求Body,不携带签名名称时,signature请填null
            String body = buildRequestBody(sender, receiver, templateId, templateParas, statusCallBack, signature);
            if (null == body || body.isEmpty()) {
                System.out.println("body is null.");
                return error("body is null.");
            }

            //请求Headers中的X-WSSE参数值
            String wsseHeader = buildWsseHeader(appKey, appSecret);
            if (null == wsseHeader || wsseHeader.isEmpty()) {
                System.out.println("wsse header is null.");
                return error("wsse header is null.");
            }

            //如果JDK版本是1.8,可使用如下代码
            CloseableHttpClient client = HttpClients.custom()
                    .setSSLContext(new SSLContextBuilder().loadTrustMaterial(null,
                            (x509CertChain, authType) -> true).build())
                    .setSSLHostnameVerifier(new DefaultHostnameVerifier())
                    .build();

            //请求方法POST
            HttpResponse response = client.execute(RequestBuilder.create("POST")
                    .setUri(url)
                    .addHeader(HttpHeaders.CONTENT_TYPE, "application/x-www-form-urlencoded")
                    .addHeader(HttpHeaders.AUTHORIZATION, AUTH_HEADER_VALUE)
                    .addHeader("X-WSSE", wsseHeader)
                    .setEntity(new StringEntity(body)).build());

            //打印响应头域信息
            System.out.println(response.toString());
            //打印响应消息实体
            System.out.println(EntityUtils.toString(response.getEntity()));
            return success();
        }
    }

    /**
     * 构造请求Body体
     *
     * @param sender
     * @param receiver
     * @param templateId
     * @param templateParas
     * @param statusCallbackUrl
     * @param signature         | 签名名称,使用国内短信通用模板时填写
     * @return
     */
    static String buildRequestBody(String sender, String receiver, String templateId, String templateParas,
                                   String statusCallbackUrl, String signature) {
        if (null == sender || null == receiver || null == templateId || sender.isEmpty() || receiver.isEmpty()
                || templateId.isEmpty()) {
            System.out.println("buildRequestBody(): sender, receiver or templateId is null.");
            return null;
        }
        List<NameValuePair> keyValues = new ArrayList<NameValuePair>();

        keyValues.add(new BasicNameValuePair("from", sender));
        keyValues.add(new BasicNameValuePair("to", receiver));
        keyValues.add(new BasicNameValuePair("templateId", templateId));
        if (null != templateParas && !templateParas.isEmpty()) {
            keyValues.add(new BasicNameValuePair("templateParas", templateParas));
        }
        if (null != statusCallbackUrl && !statusCallbackUrl.isEmpty()) {
            keyValues.add(new BasicNameValuePair("statusCallback", statusCallbackUrl));
        }
        if (null != signature && !signature.isEmpty()) {
            keyValues.add(new BasicNameValuePair("signature", signature));
        }

        return URLEncodedUtils.format(keyValues, Charset.forName("UTF-8"));
    }

    /**
     * 构造X-WSSE参数值
     *
     * @param appKey
     * @param appSecret
     * @return
     */
    static String buildWsseHeader(String appKey, String appSecret) {
        if (null == appKey || null == appSecret || appKey.isEmpty() || appSecret.isEmpty()) {
            System.out.println("buildWsseHeader(): appKey or appSecret is null.");
            return null;
        }
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'");
        String time = sdf.format(new Date());
        String nonce = UUID.randomUUID().toString().replace("-", "");
        byte[] passwordDigest = DigestUtils.sha256(nonce + time + appSecret);
        String hexDigest = Hex.encodeHexString(passwordDigest);
        String passwordDigestBase64Str = Base64.getEncoder().encodeToString(hexDigest.getBytes());
        return String.format(WSSE_HEADER_FORMAT, appKey, passwordDigestBase64Str, nonce, time);
    }

    @PostMapping("/check")
    @Log(title="检测短信api", businessType = BusinessType.OTHER)
    public String check(@RequestBody Map<String,String> param) {
        String checkCode = param.get("token");
        Map<String, Object> map = new HashMap<>();
        map.put("secret", "6Lc2gP8fAAAAADhnazdU97DfzK2wWdv1QalaYeJu");
        map.put("response", checkCode);
        return HttpRequest.post("https://www.recaptcha.net/recaptcha/api/siteverify")
                            .form(map)
                            .timeout(20000)
                            .charset("UTF-8")
                            .execute().body(); }
}


